package cz.muni.pa165.account.service;

import cz.muni.pa165.account.AccountTestBase;
import cz.muni.pa165.account.MockedEntities;
import cz.muni.pa165.core.exception.ResourceNotFoundException;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;

@SpringBootTest
class AccountServiceTest extends AccountTestBase {

    @Test
    void findByIdExisting() {
        Assertions.assertSame(
            MockedEntities.JOHN_WICK,
            service.findById(MockedEntities.JOHN_WICK.getId())
        );
    }

    @Test
    void findByIdNotExisting() {
        Assertions.assertThrows(
            ResourceNotFoundException.class,
            () -> service.findById(2L)
        );
    }

    @Test
    void findAll() {
        Assertions.assertEquals(
            new PageImpl<>(MockedEntities.ACCOUNTS),
            service.findAll(PageRequest.of(0, 10))
        );
    }

    @Test
    void create() {
        Assertions.assertSame(
            MockedEntities.JOHN_WICK,
            service.create(MockedEntities.JOHN_WICK)
        );
    }

    @Test
    void update() {
        Assertions.assertSame(
            MockedEntities.JOHN_WICK,
            service.update(MockedEntities.JOHN_WICK)
        );
        Mockito.verify(repository).findById(MockedEntities.JOHN_WICK.getId());
    }

    @Test
    void delete() {
        service.delete(MockedEntities.JOHN_WICK.getId());
        Mockito.verify(repository).deleteById(MockedEntities.JOHN_WICK.getId());
    }
}
