package cz.muni.pa165.movie.service;

import cz.muni.pa165.core.exception.ResourceNotFoundException;
import cz.muni.pa165.movie.MockedEntities;
import cz.muni.pa165.movie.MovieTestBase;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;

@SpringBootTest
class MovieServiceTest extends MovieTestBase {

    @Test
    void findByIdExisting() {
        Assertions.assertSame(
            MockedEntities.PIRATES_OF_THE_CARIBBEAN,
            service.findById(MockedEntities.PIRATES_OF_THE_CARIBBEAN.getId())
        );
    }

    @Test
    void findByIdNotExisting() {
        Assertions.assertThrows(
            ResourceNotFoundException.class,
            () -> service.findById(2L)
        );
    }

    @Test
    void findAll() {
        Assertions.assertEquals(
            new PageImpl<>(MockedEntities.MOVIES),
            service.findAll(PageRequest.of(0, 1))
        );
    }

    @Test
    void create() {
        Assertions.assertSame(
            MockedEntities.PIRATES_OF_THE_CARIBBEAN,
            service.create(MockedEntities.PIRATES_OF_THE_CARIBBEAN)
        );
    }

    @Test
    void update() {
        Assertions.assertSame(
            MockedEntities.PIRATES_OF_THE_CARIBBEAN,
            service.update(MockedEntities.PIRATES_OF_THE_CARIBBEAN)
        );
        Mockito
            .verify(repository)
            .findById(MockedEntities.PIRATES_OF_THE_CARIBBEAN.getId());
    }

    @Test
    void delete() {
        service.delete(MockedEntities.PIRATES_OF_THE_CARIBBEAN);
        Mockito
            .verify(repository)
            .delete(MockedEntities.PIRATES_OF_THE_CARIBBEAN);
    }
}
